@@ -5,55 +5,61 @@ function(model,...) UseMethod("cumres")
   cumres.glm(model,...)
 }
 
-`cumres.glm` <-
-  cc <- 
-  function(model,
+`cumres.glm` <- function(model,
          variable=c("predicted",colnames(model.matrix(model))),
          data=data.frame(model.matrix(model)), 
-         R=500, b=0, plots=min(R,50),breakties=1e-12,
+         R=1000, b=0, plots=min(R,50),breakties=1e-12,
          seed=round(runif(1,1,1e9)),...) {
   dginv <- function(z) 1
   a.phi <- 1
   switch(class(model)[1],
          lm = {
            a.phi <- summary(model)$sigma^2
+           h <- function(z) 1
          },
          glm = {
            f <- family(model)
-           if (f$family=="gaussian")
-             stop("Refit model using 'lm'")
-           
-           if (f$family=="poisson" & f$link=="log") {
-             dginv <- function(z) exp(z)
-           } else if (f$family=="binomial" & f$link=="logit") {
-             dginv <- function(z) exp(-z)/(1+exp(-z))^2
-           } else {
-             stop("Unsupported model!")
+           if (f$family=="gaussian") {
+             a.phi <- summary(model)$dispersion
            }
+           ## stop("Refit model using 'lm'")           
+
+           g <- f$linkfun
+           ginv <- f$linkinv
+           dginv <- f$mu.eta ## D[linkinv]
+           ##dg <- function(x) 1/dginv(g(x)) ## Dh^-1 = 1/(h'(h^-1(x)))
+           canonf <- do.call(f$family,list())           
+           caninvlink <- canonf$linkinv
+           canlink <- canonf$linkfun
+           Dcaninvlink <- canonf$mu.eta           
+           Dcanlink <- function(x) 1/Dcaninvlink(canlink(x))
+           ##gmu <- function(x) g(caninvlink(x))
+           ##invgmu <- function(z) canlink(ginv(z))
+           h <- function(z) Dcanlink(ginv(z))*dginv(z)                                
          },
          stop("Unsupported model!"))
 
   response <- all.vars(formula(model))[1]
   X <- model.matrix(model)
-  n <- nrow(X)
-  
-  r <- residuals(model, type="response") ## g^{-1}(Xb)
-  yhat <- predict(model, type="response") ## y-g^{-1}(Xb)
-##  Xbeta <- predict(model, type="link") ## X*b
-  Xbeta <- X%*%coef(model)
-
+  n <- nrow(X)  
+  r <- residuals(model, type="response") ## y-g^{-1}(Xb)
+  yhat <- predict(model, type="response") ## g^{-1}(Xb)
+  ##  Xbeta <- predict(model, type="link") ## X*b
+  beta <- coef(model)
+  if(any(is.na(beta))) stop("Over-parametrized model")
+  Xbeta <- X%*%beta
+ 
   etaraw <- (as.numeric(dginv(Xbeta))*X)
-  
   hatW.MC <- function(x) {
     myorder <- order(x)
     x <- x[myorder]
     Ii <- vcov(model)
-    Score <- (X*r)/a.phi
-    beta.iid <- Ii%*%t(Score[myorder,,drop=FALSE])
-    ##    print(beta.iid)
+    ##    S <- (X*(as.vector(h(Xbeta))*r))/a.phi
+    A <- as.vector(h(Xbeta)*r)/a.phi 
+    S <- apply(X,2,function(x) x*A)
+    beta.iid <- Ii%*%t(S[myorder,,drop=FALSE])
     r0 <- r[myorder]
     D0 <- etaraw[myorder,,drop=FALSE]
-    ##    print(head(t(beta.iid)))
     Wfun <- "W2"
     if (b!=0) { Wfun <- "W" }
     output <- .C(Wfun,
@@ -61,7 +67,7 @@ function(model,...) UseMethod("cumres")
                  b=as.double(b), ## Moving average parameter
                  n=as.integer(n), ## Number of observations
                  npar=as.integer(nrow(Ii)),
-                 xdata=as.double(x), ## Observations to cummulate after 
+                 xdata=as.double(x), ## Observations to cumulate after 
                  rdata=as.double(r0), ## Residuals (one-dimensional)
                  betaiiddata=as.double(beta.iid), ## Score-process
                  etarawdata=as.double(D0), ## Eta (derivative of terms in cummulated process W)
@@ -78,12 +84,12 @@ function(model,...) UseMethod("cumres")
     return(list(output=output,x=x))
   }
   
-  
   if (!is.na(match(response, variable))) variable[match(response, variable)] <- "predicted"
   variable <- unique(variable)
-  UsedData <- data[,na.omit(match(variable, names(data))),drop=FALSE]
-  myvars <- names(UsedData)[apply(UsedData,2,function(x) length(unique(x))>2)] ## Only consider variables with more than two levels
+  UsedData <- X[,na.omit(match(variable, colnames(X))),drop=FALSE]
+  myvars <- colnames(UsedData)[apply(UsedData,2,function(x) length(unique(x))>2)] ## Only consider variables with more than two levels
   if ("predicted"%in%variable) myvars <- c("predicted",myvars)
+
   
   untie <- runif(n,0,breakties)
   
@@ -99,9 +105,9 @@ function(model,...) UseMethod("cumres")
   for (v in myvars) {
     x <- NULL
     if (v=="predicted") {
-      x <- yhat
-    } else if (v %in% names(data)) {
-      x <- data[,v]       
+      x <- yhat ## Xbeta
+    } else if (v %in% colnames(X)) {
+      x <- X[,v]       
     }
     if (!is.null(x)) {
       UsedVars <- c(UsedVars, v)
